<?php
  /**
 * This file is part of a project maintained at  NC State University.
 *
 * More information is included in the project README.md.
 *
 * (c) NC State University
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace NCSU\Auth\Adapter;

use Exception;
use NCSU\Auth\AuthResult;
use NCSU\Auth\Exception\RuntimeException;
use NCSU\Auth\Http\Request;

/**
 * Authentication adapter for Shibboleth 2.5
 * @see https://shib.ncsu.edu/
 *
 * @author Dustin Wheeler <mdwheele@ncsu.edu>
 */
final class ShibAuthAdapter implements AdapterInterface
{
    const IDP_TEMPLATE = '%s/Shibboleth.sso/Login?target=%s';

    protected $request;

    public function __construct(Request $request)
    {
        $this->request = $request;
    }

    /**
     * Attempts to authenticate and returns user to client page.
     *
     * @throws RuntimeException
     * @return \NCSU\Auth\AuthResult
     */
    public function authenticate()
    {
        if ( ! $this->hasShibbolethSession()) {
            header(sprintf("Location: %s", $this->resolveLoginUriFromTemplate()));
            exit();
        }

        // Technically, this is not specific enough. This is being done as a non-BC change
        // to allow previous WRAP users to not have to re-identify users with correct scopes.
        $wrapIdentifier = $this->getShibbolethIdentifierFromEnvironment();

        if (empty($wrapIdentifier)) {
            return new AuthResult(AuthResult::FAILURE, null);
        }

        return new AuthResult(AuthResult::SUCCESS, $wrapIdentifier);
    }

    /**
     * @return string
     */
    private function resolveLoginUriFromTemplate()
    {
        return sprintf(ShibAuthAdapter::IDP_TEMPLATE, $this->request->getHost(), $this->request->getCurrentUri());
    }

    private function hasShibbolethSession()
    {
        return isset($this->request->server['Shib-Session-ID']) or
            isset($this->request->server['Shib_Session_ID']) or
            isset($this->request->server['REDIRECT_Shib_Session_ID']);
    }

    private function getShibbolethIdentifierFromEnvironment()
    {
        if (isset($this->request->server['SHIB_UID'])) {
            return $this->request->server['SHIB_UID'];
        }

        if (isset($this->request->server['REDIRECT_SHIB_UID'])) {
            return $this->request->server['REDIRECT_SHIB_UID'];
        }

        return null;
    }

    /**
     * Returns true if user is authenticated, false otherwise
     *
     * @return bool
     */
    public function isAuthenticated()
    {
        return $this->hasShibbolethSession();
    }
}
